@@ -310,7 +310,7 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
   if ((ptr.Format >> 4) == 0x4 || (ptr.Format >> 4) == 0x5 || (ptr.Format >> 4) == 0x0)
   {
     if (ptr.Length > 4)
-      handleBlob(&tmpInput, level+1);
+      handleBlob(&tmpInput, shift, level+1);
     if ((ptr.Format >> 4) == 0x5 && ptr.Type != VSD_COLORS)
       handleStreams(&tmpInput, ptr.Type, shift, level+1);
   }
@@ -364,13 +364,13 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
 
 }
 
-void libvisio::VSDParser::handleBlob(WPXInputStream *input, unsigned level)
+void libvisio::VSDParser::handleBlob(WPXInputStream *input, unsigned shift, unsigned level)
 {
   try
   {
     m_header.level = level;
-    m_header.trailer = 0;
-    m_header.dataLength = readU32(input);
+    input->seek(shift, WPX_SEEK_SET);
+    m_header.dataLength -= shift;
     _handleLevelChange(m_header.level);
     handleChunk(input);
   }
@@ -759,18 +759,22 @@ void libvisio::VSDParser::readTextBlock(WPXInputStream *input)
 
 void libvisio::VSDParser::readGeomList(WPXInputStream *input)
 {
-  uint32_t subHeaderLength = readU32(input);
-  uint32_t childrenListLength = readU32(input);
-  input->seek(subHeaderLength, WPX_SEEK_CUR);
-  std::vector<unsigned> geometryOrder;
-  geometryOrder.reserve(childrenListLength / sizeof(uint32_t));
-  for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
-    geometryOrder.push_back(readU32(input));
-
   if (!m_shape.m_geometries.empty() && m_currentGeometryList->empty())
     m_shape.m_geometries.erase(--m_currentGeomListCount);
   m_currentGeometryList = &m_shape.m_geometries[m_currentGeomListCount++];
-  m_currentGeometryList->setElementsOrder(geometryOrder);
+
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, WPX_SEEK_CUR);
+    std::vector<unsigned> geometryOrder;
+    geometryOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      geometryOrder.push_back(readU32(input));
+
+    m_currentGeometryList->setElementsOrder(geometryOrder);
+  }
 
   // We want the collectors to still get the level information
   if (!m_isStencilStarted)
@@ -779,36 +783,44 @@ void libvisio::VSDParser::readGeomList(WPXInputStream *input)
 
 void libvisio::VSDParser::readCharList(WPXInputStream *input)
 {
-  uint32_t subHeaderLength = readU32(input);
-  uint32_t childrenListLength = readU32(input);
-  input->seek(subHeaderLength, WPX_SEEK_CUR);
-  std::vector<unsigned> characterOrder;
-  characterOrder.reserve(childrenListLength / sizeof(uint32_t));
-  for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
-    characterOrder.push_back(readU32(input));
-
-  m_shape.m_charList.setElementsOrder(characterOrder);
-
   // We want the collectors to still get the level information
   if (!m_isStencilStarted)
     m_collector->collectUnhandledChunk(m_header.id, m_header.level);
+
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, WPX_SEEK_CUR);
+    std::vector<unsigned> characterOrder;
+    characterOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      characterOrder.push_back(readU32(input));
+
+    m_shape.m_charList.setElementsOrder(characterOrder);
+  }
+
 }
 
 void libvisio::VSDParser::readParaList(WPXInputStream *input)
 {
-  uint32_t subHeaderLength = readU32(input);
-  uint32_t childrenListLength = readU32(input);
-  input->seek(subHeaderLength, WPX_SEEK_CUR);
-  std::vector<unsigned> paragraphOrder;
-  paragraphOrder.reserve(childrenListLength / sizeof(uint32_t));
-  for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
-    paragraphOrder.push_back(readU32(input));
-
-  m_shape.m_paraList.setElementsOrder(paragraphOrder);
-
   // We want the collectors to still get the level information
   if (!m_isStencilStarted)
     m_collector->collectUnhandledChunk(m_header.id, m_header.level);
+
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, WPX_SEEK_CUR);
+    std::vector<unsigned> paragraphOrder;
+    paragraphOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      paragraphOrder.push_back(readU32(input));
+
+    m_shape.m_paraList.setElementsOrder(paragraphOrder);
+  }
+
 }
 
 void libvisio::VSDParser::readPropList(WPXInputStream * /* input */)
@@ -915,21 +927,24 @@ void libvisio::VSDParser::readShapeId(WPXInputStream *input)
 
 void libvisio::VSDParser::readShapeList(WPXInputStream *input)
 {
-  uint32_t subHeaderLength = readU32(input);
-  uint32_t childrenListLength = readU32(input);
-  input->seek(subHeaderLength, WPX_SEEK_CUR);
-  std::vector<unsigned> shapeOrder;
-  shapeOrder.reserve(childrenListLength / sizeof(uint32_t));
-  for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
-    shapeOrder.push_back(readU32(input));
-
-  if (!m_isShapeStarted)
-    m_shapeList.setElementsOrder(shapeOrder);
-  else
-    m_shape.m_shapeList.setElementsOrder(shapeOrder);
-
   // We want the collectors to still get the level information
   m_collector->collectUnhandledChunk(m_header.id, m_header.level);
+
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, WPX_SEEK_CUR);
+    std::vector<unsigned> shapeOrder;
+    shapeOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      shapeOrder.push_back(readU32(input));
+
+    if (!m_isShapeStarted)
+      m_shapeList.setElementsOrder(shapeOrder);
+    else
+      m_shape.m_shapeList.setElementsOrder(shapeOrder);
+  }
 }
 
 void libvisio::VSDParser::readForeignDataType(WPXInputStream *input)
@@ -1400,21 +1415,24 @@ void libvisio::VSDParser::readNameList(WPXInputStream * /* input */)
 
 void libvisio::VSDParser::readFieldList(WPXInputStream *input)
 {
-  uint32_t subHeaderLength = readU32(input);
-  uint32_t childrenListLength = readU32(input);
-  input->seek(subHeaderLength, WPX_SEEK_CUR);
-  std::vector<unsigned> fieldOrder;
-  fieldOrder.reserve(childrenListLength / sizeof(uint32_t));
-  for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
-    fieldOrder.push_back(readU32(input));
+  if (m_header.trailer)
+  {
+    uint32_t subHeaderLength = readU32(input);
+    uint32_t childrenListLength = readU32(input);
+    input->seek(subHeaderLength, WPX_SEEK_CUR);
+    std::vector<unsigned> fieldOrder;
+    fieldOrder.reserve(childrenListLength / sizeof(uint32_t));
+    for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
+      fieldOrder.push_back(readU32(input));
 
-  m_shape.m_fields.setElementsOrder(fieldOrder);
-  m_shape.m_fields.addFieldList(m_header.id, m_header.level);
+    m_shape.m_fields.setElementsOrder(fieldOrder);
+    m_shape.m_fields.addFieldList(m_header.id, m_header.level);
+  }
 }
 
 void libvisio::VSDParser::readColours(WPXInputStream *input)
 {
-  input->seek(6, WPX_SEEK_SET);
+  input->seek(2, WPX_SEEK_CUR);
   unsigned numColours = readU8(input);
   input->seek(1, WPX_SEEK_CUR);
   m_colours.clear();
@@ -1457,7 +1475,7 @@ void libvisio::VSDParser::readFontIX(WPXInputStream *input)
 
   ::WPXBinaryData textStream;
 
-  for (int i = 0; i < m_header.dataLength + tmpAdjust; i++)
+  for (long i = 0; i < (long)(m_header.dataLength + tmpAdjust); i++)
   {
     unsigned char curchar = readU8(input);
     if (curchar == 0)
@@ -1651,11 +1669,17 @@ void libvisio::VSDParser::readParaIX(WPXInputStream *input)
 void libvisio::VSDParser::readFillAndShadow(WPXInputStream *input)
 {
   Colour colourFG = _colourFromIndex(readU8(input));
-  input->seek(3, WPX_SEEK_CUR);
-  double fillFGTransparency = (double)readU8(input) / 255.0;
+  colourFG.r = readU8(input);
+  colourFG.g = readU8(input);
+  colourFG.b = readU8(input);
+  colourFG.a = readU8(input);
+  double fillFGTransparency = (double)colourFG.a / 255.0;
   Colour colourBG = _colourFromIndex(readU8(input));
-  input->seek(3, WPX_SEEK_CUR);
-  double fillBGTransparency = (double)readU8(input) / 255.0;
+  colourBG.r = readU8(input);
+  colourBG.g = readU8(input);
+  colourBG.b = readU8(input);
+  colourBG.a = readU8(input);
+  double fillBGTransparency = (double)colourBG.a / 255.0;
   unsigned char fillPattern = readU8(input);
   input->seek(1, WPX_SEEK_CUR);
   Colour shfgc;            // Shadow Foreground Colour
