@@ -1258,6 +1258,8 @@ void libvisio::VSDXMLParserBase::readCharIX(xmlTextReaderPtr reader)
             font = VSDName(WPXBinaryData(stringValue, xmlStrlen(stringValue)), VSD_TEXT_UTF8);
           }
         }
+        if (stringValue)
+          xmlFree(stringValue);
       }
       break;
     case XML_COLOR:
@@ -1843,13 +1845,7 @@ void libvisio::VSDXMLParserBase::handleMastersEnd(xmlTextReaderPtr /* reader */)
 {
   m_isShapeStarted = false;
   if (m_extractStencils)
-  {
-    m_collector->collectShapesOrder(0, 2, m_shapeList.getShapesOrder());
-    _handleLevelChange(0);
-    m_shapeList.clear();
-    m_isPageStarted = false;
-    m_collector->endPage();
-  }
+    m_collector->endPages();
   else
     m_isStencilStarted = false;
 }
@@ -1869,7 +1865,10 @@ void libvisio::VSDXMLParserBase::handleMasterEnd(xmlTextReaderPtr /* reader */)
   m_isPageStarted = false;
   if (m_extractStencils)
   {
+    m_collector->collectShapesOrder(0, 2, m_shapeList.getShapesOrder());
     _handleLevelChange(0);
+    m_shapeList.clear();
+    m_isPageStarted = false;
     m_collector->endPage();
   }
   else
